# encoding: utf-8

require 'easy_html_generator/version'

# just require bootsrap and it will be in the sass path
require 'bootstrap-sass'
require 'bundler'
require 'colorize'
require 'rack'
require 'rack/contrib'
require 'thin'

# main module loads projects and dispatches rack requests
module EasyHtmlGenerator
  EHG_WORKSPACE_PATH =
    File.expand_path File.join(File.dirname(__FILE__), '..')

  EHG_SRC_PATH            = File.join(EHG_WORKSPACE_PATH, 'src')
  EHG_SHARED_PATH         = File.join(EHG_SRC_PATH, 'shared')
  EHG_SHARED_HELPER_PATH  = File.join(EHG_SHARED_PATH, 'lib', 'helper')
  EHG_SHARED_GENERATORS_PATH = File.join(EHG_SHARED_PATH, 'lib', 'generators')
  EHG_SHARED_STYLES_PATH  = File.join(EHG_SHARED_PATH, 'assets', 'styles')

  WORKSPACE_PATH          = Bundler.root
  SRC_PATH                = File.join(WORKSPACE_PATH, 'src')
  DIST_PATH               = File.join(WORKSPACE_PATH, 'dist')
  SHARED_PATH             = File.join(SRC_PATH, 'shared')
  SHARED_HELPER_PATH      = File.join(SHARED_PATH, 'lib', 'helper')
  SHARED_GENERATORS_PATH  = File.join(SHARED_PATH, 'lib', 'generators')
  SHARED_STYLES_PATH      = File.join(SHARED_PATH, 'assets', 'styles')

  CREATE_ON_INIT          = [
    SRC_PATH,
    DIST_PATH
  ]

  COPY_ON_INIT = {
    EHG_SHARED_PATH            => SHARED_PATH,
    "#{EHG_SRC_PATH}/template" => "#{SRC_PATH}/demo"
  }

  Dir[File.dirname(__FILE__) + '/easy_html_generator/*.rb']
    .each { |file| require file }

  def self.dispatch(request)
    path         = request.env['REQUEST_PATH'] || request.env['PATH_INFO']
    cleaned_path = clean_path path
    project_name = project_name_by_path path

    project = Workspace.project_by_name project_name

    return path unless project

    project.generate if project.should_generate_for? cleaned_path

    # return absolute dist path
    File.join(project.dist_path, cleaned_path)
  end

  def self.project_name_by_path(path)
    path = path.slice(1..-1) if path.start_with? '/'
    path = path.split('/').first if path.include? '/'
    path
  end

  def self.clean_path(path)
    path = path.slice(1..-1) if path.start_with? '/'
    path.sub!("#{project_name_by_path(path)}/", '')
    path
  end

  def self.rack_app
    Rack::Builder.new do
      use Rack::ShowExceptions
      use EasyHtmlGenerator::RackDispatcher
      use Rack::Deflater
      use Rack::ContentLength
      use Rack::StaticCache, urls: [''], root: EasyHtmlGenerator::DIST_PATH
      use Rack::Static,
          urls: [''],
          root: EasyHtmlGenerator::DIST_PATH,
          index: 'index.html'

      run -> (_env) do
        [404, { 'Content-Type' => 'text/plain' }, ['404 File not found!']]
      end
    end
  end

  def self.start_server(host, port)
    config = { Host: host,
               Port: port }

    Thin::Logging.debug = true
    Rack::Handler.get('thin').run(rack_app, config)
  end
end
