# encoding: utf-8
require 'digest/md5'

# this class handles file checksums
class EasyHtmlGenerator::Checksum
  @file_checksums = {}

  def self.checksum(input)
    Digest::MD5.hexdigest(input)
  end

  def self.file_checksum(file)
    return array_checksum(file) if file.is_a? Array
    return array_checksum(Dir["#{folder}/**/*"]) if File.directory? file
    checksum(File.read(file))
  end

  def self.array_checksum(files)
    tmp = ''
    files.each { |file| tmp += File.read(file) if File.file? file }
    checksum(tmp)
  end

  def self.file_changed?(file, context = '')
    key = build_key(file, context)

    return true unless @file_checksums.key? key

    @file_checksums[key] != file_checksum(file)
  end

  def self.folder_changed?(folder, context = '')
    key = build_key(folder, context)

    return true unless @file_checksums.key? key

    @file_checksums[key] != folder_checksum(folder)
  end

  def self.store_file(file, context = '')
    key = build_key(file, context)

    @file_checksums[key] = file_checksum(file)
  end

  def self.invalidate_file(file, context = '')
    key = build_key(file, context)

    @file_checksums.delete(key)
  end

  def self.invalidate_all()
    @file_checksums = {}
  end

  def self.build_key(file, context = '')
    "#{context}.#{file}"
  end
end
