# encoding: utf-8

# this class handles multiple generators for a project
class EasyHtmlGenerator::Generators
  def initialize(project)
    @project    = project
    @generators = load_generators
  end

  def generate
    @generators.each do |_index, generator|
      generator.generate
    end
  end

  def generator_class_by_name(name)
    name = name.split('_').map { |n| capitalize(n) }.join('::')
    capitalize(name) # if there is no _

    "EasyHtmlGenerator::Generator::#{name}"
  end

  private

  def capitalize(input)
    input.slice(0, 1).capitalize + input.slice(1..-1)
  end

  def load_generators
    Dir[@project.src_path_to(:generators, '**/*.rb'),
        EasyHtmlGenerator::SHARED_GENERATORS_PATH + '/**/*.rb']
      .each { |file| load file }

    result = {}
    @project.config.generators.each.map do |generator|
      name, config = generator.first
      result[name] = generator_by_name(name.to_s, config)
    end
    result
  end

  def generator_by_name(name, config)
    clazz = Object.const_get generator_class_by_name(name)

    clazz.new(@project, config)
  end

  def method_missing(name, *args)
    return super.method_missing name unless @generators.key? name

    @generators[name]
  end
end
