# encoding: utf-8
require 'fileutils'

# this class creates project instances
class EasyHtmlGenerator::Workspace
  @project_instances = {}

  def self.projects
    if @project_instances.empty?
      project_folders.each do |path|
        next if path.include? 'shared'
        p    = project_instance_from_path path
        @project_instances[p.name.to_sym] = p
      end
    end

    @project_instances
  end

  def self.project_folders
    Dir[File.join(EasyHtmlGenerator::SRC_PATH, '*')]
      .reject { |o| !File.directory?(o) }
      .map { |o| File.expand_path(o) }
  end

  def self.create_project(name)
    FileUtils.cp_r(
      File.join(EasyHtmlGenerator::EHG_SRC_PATH, 'template'),
      File.join(EasyHtmlGenerator::SRC_PATH, name))
  end

  def self.init
    EasyHtmlGenerator::CREATE_ON_INIT.each do |dir|
      FileUtils.mkdir_p dir
    end

    EasyHtmlGenerator::COPY_ON_INIT.each do |src, target|
      FileUtils.cp_r src, target
    end
  end

  def self.clean_dist_folder(project=nil)
    if project.nil?
      paths = Dir["#{EasyHtmlGenerator::DIST_PATH}/*"]
    else
      project_instance = project_by_name project
      raise LoadError "no such project to load -- #{project}"
      paths = project_instance.dist_path
    end
    FileUtils.rm_rf paths
  end

  def self.generate(project=nil)
    if project.nil?
      projects_to_generate = projects
    else
      project_instance = project_by_name project
      raise LoadError "no such project to load -- #{project}"
      projects_to_generate = {project => project_instance}
    end

    projects_to_generate.each { |_name, project| project.generate }
  end

  def self.project_instance_from_path(path)
    name = path.split('/').last.sub('/', '')

    config_file = "#{EasyHtmlGenerator::SHARED_PATH}/project.yml"
    config_file = "#{path}/project.yml" if File.exist? "#{path}/project.yml"

    EasyHtmlGenerator::Project.new(
      name,
      "#{EasyHtmlGenerator::SRC_PATH}/#{name}",
      "#{EasyHtmlGenerator::DIST_PATH}/#{name}",
      config_file
    )
  end

  def self.project_by_name(name)
    projects[name.to_sym]
  end
end
